@@ -41,6 +41,12 @@
 - (void)setAutoDate:(float)interval;
 - (IBAction) aboutClick: (id)sender;
 
+// Start timers
+- (void)clockTick;
+- (void)colorTick;
+- (void)dateTick;
+
+
 /* Bindable Properties:
      hourStyle
      timeStyle
