@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991-2006 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2007 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -22,46 +22,146 @@
 //#define BE_QUARTZY
 
 @interface DaliClockView (ForwardDeclarations)
++ (NSUserDefaultsController *)userDefaultsController;
++ (void)setUserDefaultsController:(NSUserDefaultsController *)ctl;
 - (void)setForeground:(NSColor *)fg background:(NSColor *)bg;
 - (void)clockTick;
 - (void)colorTick;
+- (void)dateTick;
 @end
 
+
 @implementation DaliClockView
 
-/* This is called when the View is created (but pretty early: there are
-   some things you can't do from here...)
- */
-- (id)initWithFrame:(NSRect)frameRect
-{
-  if ((self = [super initWithFrame:frameRect]) != nil) {
-    
-    memset (&config, 0, sizeof(config));
+static NSUserDefaultsController *staticUserDefaultsController = 0;
+
++ (NSUserDefaultsController *)userDefaultsController
+{
+  return staticUserDefaultsController;
+}
+
+
++ (void)initializeDefaults:(NSUserDefaultsController *)controller
+{
+  static BOOL initialized_p = NO;
+  if (initialized_p)
+    return;
+  initialized_p = YES;
+
+  staticUserDefaultsController = controller;
+  [controller retain];
+
+  /* Set the defaults for all preferences handled by DaliClockView.
+     (AppController or DaliClockSaverView handles other preferences).
+
+     Depending on class-initialization order, another class (e.g.
+     AppController) might have already put defaults in the preferences
+     object, so make sure we add ours instead of overwriting it all.
+   */
+  NSColor *deffg = [NSColor  blueColor];
+  NSColor *defbg = [NSColor colorWithCalibratedHue:0.50
+                                        saturation:1.00
+                                        brightness:0.70   // cyan, but darker
+                                             alpha:0.30]; // and translucent
+  NSDate *defdate = [NSDate dateWithTimeIntervalSinceNow:
+                     (NSTimeInterval) 60 * 60 * 12];  // 12 hours from now
+
+  NSDictionary* defaults = [NSDictionary dictionaryWithObjectsAndKeys:
+    @"0", @"hourStyle",
+    @"0", @"timeStyle",
+    @"0", @"dateStyle",
+    [NSArchiver archivedDataWithRootObject:deffg], @"initialForegroundColor",
+    [NSArchiver archivedDataWithRootObject:defbg], @"initialBackgroundColor",
+    @"10.0", @"cycleSpeed",
+    @"NO", @"usesCountdownTimer",
+    defdate,@"countdownDate",
+    nil];
+
+  NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:100];
+  [dict addEntriesFromDictionary: [controller initialValues]];
+  [dict addEntriesFromDictionary:defaults];
+  [controller setInitialValues:dict];
+  [[controller defaults] registerDefaults:dict];
+}
+
+
+- (void)bindPreferences
+{
+  NSUserDefaultsController *controller = staticUserDefaultsController;
+
+  [self    bind:@"hourStyle"
+       toObject:controller
+    withKeyPath:@"values.hourStyle" options:nil];
+  [self    bind:@"timeStyle"
+       toObject:controller
+    withKeyPath:@"values.timeStyle"
+        options:nil];
+  [self    bind:@"dateStyle"
+       toObject:controller
+    withKeyPath:@"values.dateStyle"
+        options:nil];
+  [self    bind:@"cycleSpeed"
+       toObject:controller
+    withKeyPath:@"values.cycleSpeed"
+        options:nil];
+  [self    bind:@"usesCountdownTimer"
+       toObject:controller
+    withKeyPath:@"values.usesCountdownTimer"
+        options:nil];
+  [self    bind:@"countdownDate"
+       toObject:controller
+    withKeyPath:@"values.countdownDate"
+        options:nil];
+
+  NSDictionary *colorBindingOptions =
+    [NSDictionary dictionaryWithObject:@"NSUnarchiveFromData"
+                                forKey:NSValueTransformerNameBindingOption];
+  [self    bind:@"initialForegroundColor"
+       toObject:controller
+    withKeyPath:@"values.initialForegroundColor"
+        options:colorBindingOptions];
+  [self    bind:@"initialBackgroundColor"
+       toObject:controller
+    withKeyPath:@"values.initialBackgroundColor"
+        options:colorBindingOptions];
+}
+
+
+- (id)initWithFrame:(NSRect)rect
+{
+  self = [super initWithFrame:rect];
+  if (! self) return nil;
+
+  memset (&config, 0, sizeof(config));
+
+  img = [[[NSImage alloc] init] retain];
+  if (! img) abort();
+
+  config.max_fps = 12;
+
+  [self setOwnWindow:YES];  // by default, we may change window background
+
+  // Tell the color selector widget to show the "opacity" slider.
+  [[NSColorPanel sharedColorPanel] setShowsAlpha:YES];
+
+  // initialize the fonts and bitmaps
+  [self setFrameSize:[self frame].size];
+
+  [self clockTick];
+  [self colorTick];
+
+  [self bindPreferences];
 
-    img = [[NSImage alloc] init];
-    if (! img) abort();
-    
-    config.max_fps = 12;
-    
-    // Tell the color selector widget to show the "opacity" slider.
-    [[NSColorPanel sharedColorPanel] setShowsAlpha:YES];
-    
-    // initialize the fonts and bitmaps
-    [self setFrameSize:[self frame].size];
-    
-    [self clockTick];
-    [self colorTick];
-  }
   return self;
 }
 
+
 -(oneway void)dealloc
 {
   [img release];
   [super dealloc];
 }
 
-
 /* Called when the user starts interactively resizing the window.
  */
 - (void)viewWillStartLiveResize
@@ -88,31 +188,31 @@
   // already on the window instead, which reduces flicker when the
   // target bitmap size shifts over a boundary).
   if ([self inLiveResize])  return;
-  
+
   int ow = config.width;
   int oh = config.height;
-  
+
    config.width  = newSize.width * 2;   // use the next-larger bitmap
    config.height = newSize.height * 2;
 //  config.width = 1280;    // always use the biggest font image
 //  config.height = 1024;
-  
+
   render_bitmap_size (&config, &config.width, &config.height);
-  
+
   if (config.render_state && (ow == config.width && oh == config.height))
     return;  // nothing to do
-  
+
   // When the window is resized, re-create the bitmaps for the largest
   // font that will now fit in the window.
-  //  
+  //
   if (config.bitmap) free (config.bitmap);
   config.bitmap = calloc (1, config.height * (config.width << 3));
   if (! config.bitmap) abort();
-  
+
   if (pixmap) free (pixmap);
   pixmap = calloc (1, config.height * config.width * 4);
   if (! pixmap) abort();
-  
+
   if (config.render_state)
     render_free (&config);
   render_init (&config);
@@ -191,24 +291,37 @@ FAIL:
   fromrect.origin.x = fromrect.origin.y = 0;
   fromrect.size.width = config.width;
   fromrect.size.height = config.height;
-  
+
+  if (config.width <= 0 || config.height <= 0) abort();
+
+  float img_aspect = (float) config.width / (float) config.height;
+  float win_aspect = framerect.size.width / (float) framerect.size.height;
+
   // Scale the image to fill the window without changing its aspect ratio.
   //
-  if (((float) framerect.size.width / (float) framerect.size.height) >
-      ((float) config.width / (float) config.height)) {
+  if (win_aspect > img_aspect) {
     torect.size.height = framerect.size.height;
-    torect.size.width = (framerect.size.height *
-                          ((float) config.width / (float) config.height));
+    torect.size.width  = framerect.size.height * img_aspect;
   } else {
-    torect.size.width = framerect.size.width;
-    torect.size.height = (framerect.size.width /
-                          ((float) config.width /(float) config.height));
+    torect.size.width  = framerect.size.width;
+    torect.size.height = framerect.size.width / img_aspect;
+  }
+
+  // The animation slows down a lot if we use truly gigantic numbers,
+  // so limit the number size in screensaver-mode.
+  //
+  if (constrainSizes) {
+    int maxh = (config.time_mode == SS ? 512 : /*256*/ 200);
+    if (torect.size.height > maxh) {
+      torect.size.height = maxh;
+      torect.size.width  = maxh * img_aspect;
+    }
   }
 
   // put a margin between the numbers and the edge of the window.
   torect.size.width  *= 0.95;  // 5% horizontally
   torect.size.height *= 0.80;  // 20% vertically
-  
+
 //  torect.size = fromrect.size; // #### debugging: don't scale
 
   // center it in the window
@@ -216,9 +329,8 @@ FAIL:
   torect.origin.x = (framerect.size.width  - torect.size.width ) / 2;
   torect.origin.y = (framerect.size.height - torect.size.height) / 2;
 
-  
 # ifndef BE_QUARTZY
-  
+
   /* This is the straightforward Aqua way of doing things.
      It's simple, but I think it might be too slow.
      (It relies on the 32bpp pixmap created in colorizePixmap
@@ -260,12 +372,17 @@ FAIL:
  */
 #endif
   if (! imgrep) abort();
-  
+
   [img setSize:[imgrep size]];
   [img setScalesWhenResized:NO];
 
-//  [[self window] setOpaque:NO];
-  [[self window] setBackgroundColor:bg];
+  if (ownWindow) {
+    // [[self window] setOpaque:NO];
+    [[self window] setBackgroundColor:bg];
+  } else {
+    [bg set];
+    [NSBezierPath fillRect:framerect];
+  }
 
   [img addRepresentation:imgrep];
   [img drawInRect:torect
@@ -274,19 +391,19 @@ FAIL:
          fraction:1.0];
   [img removeRepresentation:imgrep];
   [imgrep release];
-  
+
 # else /* BE_QUARTZY */
-  
+
   /* This uses lower level Quartz/CoreGraphics calls to do the drawing.
      It might be more efficient, but it's way more complicated...
    */
-  
+
   float fgr, fgg, fgb, fga;
   float bgr, bgg, bgb, bga;
   [fg getRed:&fgr green:&fgg blue:&fgb alpha:&fga];
   [bg getRed:&bgr green:&bgg blue:&bgb alpha:&bga];
 
-  CGDataProviderRef provider = 
+  CGDataProviderRef provider =
     CGDataProviderCreateWithData (NULL /* info */, config.bitmap,
                                   config.height * (config.width>>3),
                                   NULL);
@@ -308,7 +425,7 @@ FAIL:
                                 NO /* shouldInterpolate */
                                 );
   CGDataProviderRelease (provider);
-  
+
   CGContextRef cgc = [[NSGraphicsContext currentContext] graphicsPort];
   CGRect bgrect, fgrect;
   bgrect.origin.x    = framerect.origin.x;
@@ -325,10 +442,10 @@ FAIL:
        we can just blast the background down as a rectangle.
      */
     CGContextSetRGBFillColor (cgc, bgr,bgg,bgb,bga);
-    if (bga < 1.0) 
+    if (bga < 1.0)
       CGContextClearRect (cgc, bgrect);
     CGContextFillRect (cgc, bgrect);
- 
+
   } else {
     /* foreground is transparent; background is solid or transparent.
        we have to draw the background with a mask so that it doesn't
@@ -337,7 +454,7 @@ FAIL:
     CGContextSaveGState (cgc);
     CGContextSetRGBFillColor (cgc, bgr,bgg,bgb,bga);
     CGContextClearRect (cgc, bgrect);
-    
+
     /* Ok, this is fucked up.  Since our "mask" bitmap is not as large as
        the window itself, and there's no way to specify a mask origin for
        the inverse mask (that is, say "draw everything *but* this mask)
@@ -359,18 +476,20 @@ FAIL:
     rects[3].origin.x = bgrect.origin.x+fgrect.origin.x+fgrect.size.width/*-1*/;
     rects[3].size.width = bgrect.size.width-fgrect.size.width-fgrect.origin.x;
     rects[3].size.height = fgrect.size.height;
+
     CGContextFillRects (cgc, rects, 4);
-    
+
     CGContextClipToMask (cgc, fgrect, bgmask);
     CGContextFillRect (cgc, fgrect);
     CGImageRelease (bgmask);
     bgmask = 0;
     CGContextRestoreGState (cgc);
   }
-  
+
   CGContextSetRGBFillColor (cgc, fgr,fgg,fgb,fga);
   CGContextClipToMask (cgc, fgrect, fgmask);
   CGContextFillRect (cgc, fgrect);
+
   CGImageRelease (fgmask);
   fgmask = 0;
 
@@ -381,7 +500,7 @@ FAIL:
 /* The code in digital.c gives us back a 1bpp bitmap (in config.bitmap)
    To actually render that in color with Aqua, we need to scale it up to
    an RGBA image (in self.pixmap) using the colors in "fg" and "bg".
-  
+
    If we're doing the hairy Quartz stuff, this isn't needed.
 */
 #ifndef BE_QUARTZY
@@ -403,7 +522,7 @@ FAIL:
   unsigned char bgg = 0;
   unsigned char bgb = 0;
   unsigned char bga = 0;
-  
+
   for (y = 0; y < config.height; y++)
   {
     for (x = 0; x < config.width; x++)
@@ -426,19 +545,18 @@ FAIL:
 #endif /* !BE_QUARTZY */
 
 
-/* When this timer goes off, we re-generate the bitmap/pixmap, 
+/* When this timer goes off, we re-generate the bitmap/pixmap,
    and mark the display as invalid.
 */
 - (void)clockTick
 {
-  if (clockTimer && [clockTimer isValid])
-  {
+  if (clockTimer && [clockTimer isValid]) {
     [clockTimer invalidate];
     clockTimer = 0;
   }
-  
+
   if (config.max_fps <= 0) abort();
-  
+
   NSWindow *w = [self window];
   if (w && ![w isMiniaturized]) {   // noop if no window yet, or if iconified.
     struct timeval now;
@@ -451,7 +569,7 @@ FAIL:
     // [[self window] invalidateShadow]; // windows with shadows flicker...
     [self setNeedsDisplay:TRUE];
   }
-  
+
   // re-schedule the timer according to current fps.
   //
   float delay = 0.9 / config.max_fps;
@@ -459,7 +577,7 @@ FAIL:
                                                 target:self
                                               selector:@selector(clockTick)
                                               userInfo:nil
-                                               repeats:NO];  
+                                               repeats:NO];
 }
 
 
@@ -468,14 +586,13 @@ FAIL:
  */
 - (void)colorTick
 {
-  if (colorTimer && [colorTimer isValid])
-  {
+  if (colorTimer && [colorTimer isValid]) {
     [colorTimer invalidate];
     colorTimer = 0;
   }
-  
+
   if (config.max_cps <= 0) return;   // cycling is turned off, do nothing
-  
+
   if ([self window]) {  // do nothing if no window yet
 
     float h, s, v, a;
@@ -494,7 +611,7 @@ FAIL:
 
     [self setForeground:fg2 background:bg2];
   }
-  
+
   /* re-schedule the timer according to current fps.
    */
   float delay = 1.0 / config.max_cps;
@@ -505,12 +622,78 @@ FAIL:
                                                repeats:NO];
 }
 
-- (void)updateCountdown {
+
+/* When this timer goes off, we switch to "show date" mode.
+ */
+- (void)dateTick
+{
+  if (dateTimer && [dateTimer isValid]) {
+    [dateTimer invalidate];
+    dateTimer = 0;
+  }
+
+  if (autoDateInterval <= 0) return;
+
+  BOOL was_on = config.display_date_p;
+  config.display_date_p = !was_on;
+
+  /* re-schedule the timer according to current fps.
+   */
+  float delay = autoDateInterval;
+  if (!was_on) delay = 1.0;
+  dateTimer = [NSTimer scheduledTimerWithTimeInterval:delay
+                                               target:self
+                                             selector:@selector(dateTick)
+                                             userInfo:nil
+                                              repeats:NO];
+}
+
+
+- (void)updateCountdown
+{
   config.countdown = (usesCountdownTimer
                       ? [countdownDate timeIntervalSince1970]
                       : 0);
 }
 
+
+/* The About button in the menu bar or preferences sheet.
+ */
+- (IBAction)aboutClick:(id)sender
+{
+  NSBundle *nsb = [NSBundle bundleForClass:[self class]];
+  NSAssert1 (nsb, @"no bundle for class %@", [self class]);
+
+  NSDictionary *info = [nsb infoDictionary];
+//  NSString *name = [info objectForKey:@"CFBundleExecutable"];
+  NSString *name = @"Dali Clock";
+  NSString *vers = [info objectForKey:@"CFBundleVersion"];
+  NSString *ver2 = [info objectForKey:@"CFBundleShortVersionString"];
+  NSString *icon = [info objectForKey:@"CFBundleIconFile"];
+
+  NSString *cred_file = [nsb pathForResource:@"Credits" ofType:@"html"];
+  NSAttributedString *cred = [[[NSAttributedString alloc]
+                                initWithPath:cred_file
+                                documentAttributes:(NSDictionary **)NULL]
+                               autorelease];
+
+  NSString *icon_file = [nsb pathForResource:icon ofType:@"icns"];
+  NSImage *iimg = [[NSImage alloc] initWithContentsOfFile:icon_file];
+
+  NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
+    name, @"ApplicationName",
+    vers, @"Version",
+    ver2, @"ApplicationVersion",
+    cred, @"Credits",
+    iimg, @"ApplicationIcon",
+    @"",  @"Copyright",        // This is in "Credits"
+    nil];
+
+  [[NSApplication sharedApplication]
+    orderFrontStandardAboutPanelWithOptions:dict];
+}
+
+
 // hint for XCode popup menu
 #pragma mark accessors
 
@@ -526,6 +709,7 @@ FAIL:
   config.twelve_hour_p = !aHourStyle;
 }
 
+
 - (int)timeStyle { return config.time_mode; }
 - (void)setTimeStyle:(int)aTimeStyle
 {
@@ -547,7 +731,12 @@ FAIL:
 {
   if (config.max_cps != aCycleSpeed) {
     config.max_cps = (int)aCycleSpeed;
-    [self colorTick];
+    if (config.max_cps < 0.0001) {
+      [self setForeground:initialForegroundColor
+               background:initialBackgroundColor];
+    } else {
+      [self colorTick];
+    }
   }
 }
 
@@ -598,4 +787,23 @@ FAIL:
   }
 }
 
+- (void)setOwnWindow:(BOOL)own_p
+{
+  ownWindow = own_p;
+}
+
+- (void)setConstrainSizes:(BOOL)constrain_p;
+{
+  constrainSizes = constrain_p;
+}
+
+- (void)setAutoDate:(float)interval
+{
+  autoDateInterval = interval;
+
+  config.display_date_p = 1;
+  [self dateTick];
+  config.display_date_p = 0;
+}
+
 @end
