@@ -1,6 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2002, 2003
- *  Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2006 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -149,35 +148,94 @@ static struct raw_number numbers_2 [] = {
   { 0, }
 };
 
-# include "zero3.xbm"
-# include "one3.xbm"
-# include "two3.xbm"
-# include "three3.xbm"
-# include "four3.xbm"
-# include "five3.xbm"
-# include "six3.xbm"
-# include "seven3.xbm"
-# include "eight3.xbm"
-# include "nine3.xbm"
-# include "colon3.xbm"
-# include "slash3.xbm"
+# include "zeroD.xbm"
+# include "oneD.xbm"
+# include "twoD.xbm"
+# include "threeD.xbm"
+# include "fourD.xbm"
+# include "fiveD.xbm"
+# include "sixD.xbm"
+# include "sevenD.xbm"
+# include "eightD.xbm"
+# include "nineD.xbm"
+# include "colonD.xbm"
+# include "slashD.xbm"
 
-static struct raw_number numbers_3 [] = {
-  { zero3_bits, zero3_width, zero3_height },
-  { one3_bits, one3_width, one3_height },
-  { two3_bits, two3_width, two3_height },
-  { three3_bits, three3_width, three3_height },
-  { four3_bits, four3_width, four3_height },
-  { five3_bits, five3_width, five3_height },
-  { six3_bits, six3_width, six3_height },
-  { seven3_bits, seven3_width, seven3_height },
-  { eight3_bits, eight3_width, eight3_height },
-  { nine3_bits, nine3_width, nine3_height },
-  { colon3_bits, colon3_width, colon3_height },
-  { slash3_bits, slash3_width, slash3_height },
+static struct raw_number numbers_D [] = {
+  { zeroD_bits, zeroD_width, zeroD_height },
+  { oneD_bits, oneD_width, oneD_height },
+  { twoD_bits, twoD_width, twoD_height },
+  { threeD_bits, threeD_width, threeD_height },
+  { fourD_bits, fourD_width, fourD_height },
+  { fiveD_bits, fiveD_width, fiveD_height },
+  { sixD_bits, sixD_width, sixD_height },
+  { sevenD_bits, sevenD_width, sevenD_height },
+  { eightD_bits, eightD_width, eightD_height },
+  { nineD_bits, nineD_width, nineD_height },
+  { colonD_bits, colonD_width, colonD_height },
+  { slashD_bits, slashD_width, slashD_height },
   { 0, }
 };
 
+# include "zeroC.xbm"
+# include "oneC.xbm"
+# include "twoC.xbm"
+# include "threeC.xbm"
+# include "fourC.xbm"
+# include "fiveC.xbm"
+# include "sixC.xbm"
+# include "sevenC.xbm"
+# include "eightC.xbm"
+# include "nineC.xbm"
+# include "colonC.xbm"
+# include "slashC.xbm"
+
+static struct raw_number numbers_C [] = {
+  { zeroC_bits, zeroC_width, zeroC_height },
+  { oneC_bits, oneC_width, oneC_height },
+  { twoC_bits, twoC_width, twoC_height },
+  { threeC_bits, threeC_width, threeC_height },
+  { fourC_bits, fourC_width, fourC_height },
+  { fiveC_bits, fiveC_width, fiveC_height },
+  { sixC_bits, sixC_width, sixC_height },
+  { sevenC_bits, sevenC_width, sevenC_height },
+  { eightC_bits, eightC_width, eightC_height },
+  { nineC_bits, nineC_width, nineC_height },
+  { colonC_bits, colonC_width, colonC_height },
+  { slashC_bits, slashC_width, slashC_height },
+  { 0, }
+};
+
+# include "zeroB.xbm"
+# include "oneB.xbm"
+# include "twoB.xbm"
+# include "threeB.xbm"
+# include "fourB.xbm"
+# include "fiveB.xbm"
+# include "sixB.xbm"
+# include "sevenB.xbm"
+# include "eightB.xbm"
+# include "nineB.xbm"
+# include "colonB.xbm"
+# include "slashB.xbm"
+
+static struct raw_number numbers_B [] = {
+  { zeroB_bits, zeroB_width, zeroB_height },
+  { oneB_bits, oneB_width, oneB_height },
+  { twoB_bits, twoB_width, twoB_height },
+  { threeB_bits, threeB_width, threeB_height },
+  { fourB_bits, fourB_width, fourB_height },
+  { fiveB_bits, fiveB_width, fiveB_height },
+  { sixB_bits, sixB_width, sixB_height },
+  { sevenB_bits, sevenB_width, sevenB_height },
+  { eightB_bits, eightB_width, eightB_height },
+  { nineB_bits, nineB_width, nineB_height },
+  { colonB_bits, colonB_width, colonB_height },
+  { slashB_bits, slashB_width, slashB_height },
+  { 0, }
+};
+
+
 #endif /* BUILTIN_FONTS */
 
 static char *window_title;
@@ -193,6 +251,7 @@ static GC window_draw_gc, window_erase_gc;
 static XColor fg_color, bg_color, bd_color;
 
 static struct frame *base_frames [10];
+static struct frame *orig_frames [6];
 static struct frame *current_frames [6];
 static struct frame *target_frames [6];
 static struct frame *clear_frame;
@@ -489,7 +548,9 @@ load_builtin_font (Screen *screen, Visual *visual, int which)
     case 0: nums = numbers_0; break;
     case 1: nums = numbers_1; break;
     case 2: nums = numbers_2; break;
-    case 3: nums = numbers_3; break;
+    case 3: nums = numbers_D; break;
+    case 4: nums = numbers_C; break;
+    case 5: nums = numbers_B; break;
     default: abort(); break;
     }
 
@@ -580,22 +641,36 @@ set_current_scanlines (struct frame *into_frame, struct frame *from_frame)
 }
 
 static void
-one_step (struct frame *current_frame, struct frame *target_frame, int tick)
+one_step (struct frame *orig_frame,
+          struct frame *current_frame,
+          struct frame *target_frame,
+          int tick)
 {
-  register struct scanline *line = &current_frame->scanlines [0];
-  register struct scanline *target = &target_frame->scanlines [0];
-  register int i = 0, x;
+  struct scanline *orig   =    &orig_frame->scanlines [0];
+  struct scanline *curr   = &current_frame->scanlines [0];
+  struct scanline *target =  &target_frame->scanlines [0];
+  int i = 0, x;
+  int ticks = 10; /* #### */
+  int tt = ticks+1-tick;
+
+  if (!orig) return;
+
   for (i = 0; i < character_height; i++)
     {
-#define STEP(field) (line->field += ((int) (target->field - line->field))  \
-                     / tick)
+# define STEP(field) \
+         (curr->field = (orig->field \
+                         + (((int) (target->field - orig->field)) \
+                            * tt / ticks)))
+
       for (x = 0; x < MAX_SEGS_PER_LINE; x++)
         {
           STEP (left [x]);
-          STEP (right [x]);
+          STEP (right[x]);
         }
-      line++;
+      orig++;
+      curr++;
       target++;
+# undef STEP
     }
 }
 
@@ -749,6 +824,8 @@ static void
 fill_pig_cache (Display *dpy, Drawable drawable, struct frame *work_frame)
 {
   int i;
+  struct frame *orig_frame = 0;
+
   /* do `[1-9]' to `0'
      We have very little to gain by caching the `[347]' to `0' transitions,
      but what the hell.
@@ -756,7 +833,8 @@ fill_pig_cache (Display *dpy, Drawable drawable, struct frame *work_frame)
   for (i = 0; i < 9; i++)
     {
       int tick;
-      set_current_scanlines (work_frame, base_frames [0]);
+      orig_frame = base_frames [0];
+      set_current_scanlines (work_frame, orig_frame);
       for (tick = 9; tick >= 0; tick--)
         {
           Pixmap p = XCreatePixmap (dpy, drawable,
@@ -767,14 +845,15 @@ fill_pig_cache (Display *dpy, Drawable drawable, struct frame *work_frame)
           flush_segment_buffer (dpy, p, pixmap_draw_gc);
           memory_pig_zeros [i] [9 - tick] = p;
           if (tick)
-            one_step (work_frame, base_frames [i+1], tick);
+            one_step (orig_frame, work_frame, base_frames [i+1], tick);
         }
     }
   /* do `[1-8]' to `[2-9]' */
   for (i = 0; i < 8; i++)
     {
       int tick;
-      set_current_scanlines (work_frame, base_frames [i+1]);
+      orig_frame = base_frames [i+1];
+      set_current_scanlines (work_frame, orig_frame);
       for (tick = 9; tick >= 0; tick--)
         {
           Pixmap p = XCreatePixmap (dpy, drawable,
@@ -785,7 +864,7 @@ fill_pig_cache (Display *dpy, Drawable drawable, struct frame *work_frame)
           flush_segment_buffer (dpy, p, pixmap_draw_gc);
           memory_pig_digits [i] [9 - tick] = p;
           if (tick)
-            one_step (work_frame, base_frames [i+2], tick);
+            one_step (orig_frame, work_frame, base_frames [i+2], tick);
         }
     }
   if (be_a_pig > 1)
@@ -794,7 +873,8 @@ fill_pig_cache (Display *dpy, Drawable drawable, struct frame *work_frame)
       {
         int tick;
         if (i == 7) continue; /* zero transitions are already done */
-        set_current_scanlines (work_frame, base_frames [i+1]);
+        orig_frame = base_frames [i+1];
+        set_current_scanlines (work_frame, orig_frame);
         for (tick = 9; tick >= 0; tick--)
           {
             Pixmap p = XCreatePixmap (dpy, drawable,
@@ -805,7 +885,7 @@ fill_pig_cache (Display *dpy, Drawable drawable, struct frame *work_frame)
             flush_segment_buffer (dpy, p, pixmap_draw_gc);
             total_oinker_digits [i] [9 - tick] = p;
             if (tick)
-              one_step (work_frame, base_frames [(i+3)%10], tick);
+              one_step (orig_frame, work_frame, base_frames [(i+3)%10], tick);
           }
       }
 }
@@ -899,11 +979,6 @@ initialize_window (Screen *screen, Window window, Bool external_p)
     int width = x_offsets [minutes_only ? 3 : 5] + character_width;
     if (do_shape || do_overlay)
       window_offset_x = window_offset_y = 0;
-    else if (wander_p)
-      {
-        window_offset_x = (WidthOfScreen (screen) - width) / 2;
-        window_offset_y = (HeightOfScreen (screen) - character_height) / 2;
-      }
     else
       {
         window_offset_x = (xgwa.width - width) / 2;
@@ -991,6 +1066,8 @@ get_resources (Screen *screen)
   if (!strcasecmp (font_name, "BUILTIN2")) use_builtin_font = 2;
   if (!strcasecmp (font_name, "BUILTIN" )) use_builtin_font = 2;
   if (!strcasecmp (font_name, "BUILTIN3")) use_builtin_font = 3;
+  if (!strcasecmp (font_name, "BUILTIN4")) use_builtin_font = 4;
+  if (!strcasecmp (font_name, "BUILTIN5")) use_builtin_font = 5;
 #endif /* BUILTIN_FONTS */
 
   buf = get_string_resource ("mode", "Mode");
@@ -1153,6 +1230,9 @@ run_digital (Screen *screen, Window window, Bool external_p)
                   character_height + 1);
   draw_colon (dpy, pixmap, window);
 
+  for (i = 0; i < 6; i++)
+    orig_frames [i] = current_frames [i];
+
   while (1)
     {
       int n, tick;
@@ -1167,6 +1247,10 @@ run_digital (Screen *screen, Window window, Bool external_p)
       if (odate != display_date)
         draw_colon (dpy, pixmap, window);
 
+      for (n = 0; n < ndigits; n++)
+        if (target_frames [n])
+          orig_frames [n] = target_frames [n];
+
       for (n = 0; n < ndigits; n++)
         if (time_digits [n] == last_time_digits [n])
           target_frames [n] = 0;
@@ -1270,7 +1354,8 @@ run_digital (Screen *screen, Window window, Bool external_p)
                                   pixmap, pixmap_draw_gc, x_offsets [j]);
                     }
                   if (tick)
-                    one_step (current_frames[j], target_frames [j], tick);
+                    one_step (orig_frames[j], current_frames[j], 
+                              target_frames[j], tick);
                 }
             }
           /* sends up to 1k in non-pig mode */
@@ -1436,13 +1521,13 @@ fill_borders (Display *dpy, Window window, GC gc,
     {
       rects[i].x = 0;
       rects[i].y = 0;
-      rects[i].width = ~0;
+      rects[i].width = 0x3FFF;
       rects[i].height = y;
       i++;
-      rects[i].x = x;
+      rects[i].x = 0;
       rects[i].y = y + height;
-      rects[i].width = ~0;
-      rects[i].height = ~0;
+      rects[i].width = 0x3FFF;
+      rects[i].height = 0x3FFF;
       i++;
     }
   if (x > 0)
@@ -1450,11 +1535,11 @@ fill_borders (Display *dpy, Window window, GC gc,
       rects[i].x = 0;
       rects[i].y = y;
       rects[i].width = x;
-      rects[i].height = ~0;
+      rects[i].height = height;
       i++;
       rects[i].x = x + width;
       rects[i].y = y;
-      rects[i].width = ~0;
+      rects[i].width = 0x3FFF;
       rects[i].height = height;
       i++;
     }
