@@ -1,6 +1,6 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 
- *               2002, 2003 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
+ *               2002, 2003, 2006 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -96,6 +96,8 @@ static XrmOptionDescRec options [] = {
   { "-builtin1",        ".font",                XrmoptionNoArg, "BUILTIN1" },
   { "-builtin2",        ".font",                XrmoptionNoArg, "BUILTIN2" },
   { "-builtin3",        ".font",                XrmoptionNoArg, "BUILTIN3" },
+  { "-builtin4",        ".font",                XrmoptionNoArg, "BUILTIN4" },
+  { "-builtin5",        ".font",                XrmoptionNoArg, "BUILTIN5" },
   { "-memory",          ".memory",              XrmoptionSepArg, 0 },
   { "-sleaze-level",    ".memory",              XrmoptionSepArg, 0 },
   { "-oink",            ".memory",              XrmoptionNoArg, "Medium" },
@@ -467,14 +469,16 @@ where options include\n\
 #ifdef BUILTIN_FONTS
   fprintf (stderr, "\
   -font or -fn <font>           Name of an X font or built-in font to use.\n\
-                                There are four builtin fonts, named\n\
-                                \"BUILTIN0\", \"BUILTIN1\", \"BUILTIN2\",\n\
-                                and \"BUILTIN3\", with #2 being the default.\n\
+                                There are six builtin fonts, named\n\
+                                \"BUILTIN0\", through \"BUILTIN5\", with #2\n\
+                                being the default.\n\
   -builtin0                     Same as -font BUILTIN0.\n\
   -builtin1                     Same as -font BUILTIN1.\n\
   -builtin2                     Same as -font BUILTIN2.\n\
   -builtin                      Same as -font BUILTIN.\n\
   -builtin3                     Same as -font BUILTIN3.\n\
+  -builtin4                     Same as -font BUILTIN4.\n\
+  -builtin5                     Same as -font BUILTIN5.\n\
 ");
 #else /* !BUILTIN_FONTS */
   fprintf (stderr, "\
