@@ -1,5 +1,5 @@
 /* Dali Clock - a melting digital clock for PalmOS.
- * Copyright (c) 1991-2006 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2007 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -64,6 +64,7 @@ struct frame {
 
 /* The persistent preferences that aren't covered by system preferences.
  */
+#define PREFS_VERSION 1
 struct daliclock_prefs {
   Boolean inverted_p;
   Boolean seconds_only_p;
@@ -80,6 +81,7 @@ struct dali_color {
 /* The runtime settings (some initialized from system prefs, but changable.)
  */
 struct state {
+  WinHandle window;
   Boolean inverted_p;
   Boolean twelve_hour_time_p;
   enum date_state { DTime, DDateIn, DDate, DDateOut, DDateOut2, DDash }
@@ -92,6 +94,7 @@ struct state {
 
   enum { MMDDYY, DDMMYY, YYMMDD, YYDDMM, MMYYDD, DDYYMM } date_format;
   Boolean pen_down_p;
+  Boolean active_window_p;
 
   int tick;
 
@@ -109,7 +112,6 @@ struct state {
   UInt32 rom_version;
   Int16 win_width, win_height;
   Boolean color_p;
-  Boolean hires_p;
   struct dali_color fg, bg;
 };
 
@@ -219,23 +221,40 @@ copy_frame (struct state *state, FrameHandle from_h, FrameHandle to_h)
 }
 
 
+/* Returns the largest font that will fit on the screen.
+ */
+static struct raw_number *
+pick_font (struct state *state)
+{
+  int nn, cc;
+  struct raw_number *num;
+
+  if (state->seconds_only_p)
+    nn = 2, cc = 0;
+  else
+    nn = 6, cc = 2;
+
+# define CHECK(WHICH) \
+    num = get_raw_number_##WHICH (); \
+    if (state->win_width  * 1.1 >= ((num->width * nn) + (num->width * cc)) && \
+        state->win_height * 1.1 >= num->height) \
+      return num
+  CHECK(3);
+  CHECK(2);
+  CHECK(1);
+  CHECK(0);
+# undef CHECK
+  ErrDisplay ("Display too small for all fonts!");
+  return 0;
+}
+
+
 static Boolean
 init_numbers (struct state *state)
 {
-  struct raw_number *raw;
   int i;
-
-  if (!state->hires_p) {
-    if (!state->seconds_only_p)
-      raw = get_raw_number_0();
-    else
-      raw = get_raw_number_1();
-  } else {
-    if (!state->seconds_only_p)
-      raw = get_raw_number_2();
-    else
-      raw = get_raw_number_3();
-  }
+  struct raw_number *raw = pick_font (state);
+  if (!raw) return false;
 
   state->char_width  = raw[0].width;
   state->char_height = raw[0].height;
@@ -368,7 +387,12 @@ fill_target_digits (struct state *state)
       else if (state->display_date == DDateOut2)
         state->display_date = DDash;
 
-      switch (state->date_format)
+      if (state->seconds_only_p)
+        {
+          /* In seconds-only mode, always display dotm. */
+          state->target_digits [4] = d0; state->target_digits [5] = d1;
+        }
+      else switch (state->date_format)
         {
         case MMDDYY:
           state->target_digits [0] = m0; state->target_digits [1] = m1;
@@ -468,16 +492,36 @@ cache_frame_buffer (struct state *state)
               UInt16 rowbytes = 0;
               BmpGetDimensions (b, &bw, &bh, &rowbytes);
               frame_buffer_bytes_per_line = rowbytes;
-
-              if (bw >= state->win_width * 2 &&
-                  bh >= state->win_height * 2)
-                state->hires_p = true;
-
               state->win_width  = bw;
               state->win_height = bh;
             }
         }
     }
+
+# if 0
+  {
+    char buf[100];
+    int i = 0;
+    buf[i++] = 'D';
+    buf[i++] = '=';
+    buf[i++] = '0' + (frame_buffer_depth / 10);
+    buf[i++] = '0' + (frame_buffer_depth % 10);
+    buf[i++] = ' ';
+    buf[i++] = 'W';
+    buf[i++] = '=';
+    buf[i++] = '0' + (state->win_width / 100) % 10;
+    buf[i++] = '0' + (state->win_width / 10) % 10;
+    buf[i++] = '0' + (state->win_width % 10);
+    buf[i++] = ' ';
+    buf[i++] = 'H';
+    buf[i++] = '=';
+    buf[i++] = '0' + (state->win_height / 100) % 10;
+    buf[i++] = '0' + (state->win_height / 10) % 10;
+    buf[i++] = '0' + (state->win_height % 10);
+    buf[i++] = 0;
+    ErrDisplay (buf);
+  }
+# endif /* 0 */
 }
 #endif /* HACK_FRAMEBUFFER */
 
@@ -584,22 +628,57 @@ draw_frame (struct state *state, FrameHandle fh, int x, int y)
 }
 
 
-
 static void
-draw_background (struct state *state)
+clear_borders (struct state *state, Boolean all_p)
 {
+  int ww = state->win_width;
+  int wh = state->win_height;
+  int dw = (state->seconds_only_p
+            ? (state->char_width * 2)
+            : ((state->char_width * 6) + (state->colon_width * 2)));
+  int dh = state->char_height;
+  int dx = (ww - dw) / 2;
+  int dy = (wh - dh) / 2;
   RectangleType r;
-  r.topLeft.x = 0;
-  r.topLeft.y = 0;
-  WinGetWindowExtent (&r.extent.x, &r.extent.y);
-  if (state->inverted_p)
-    WinDrawRectangle (&r, 0);
-  else
-    WinEraseRectangle (&r, 0);
+  Boolean inverted_p = state->inverted_p;
+
+#define RECT() \
+   if (r.extent.x > 0 && r.extent.y > 0) \
+     (inverted_p ? WinDrawRectangle(&r, 0) : WinEraseRectangle (&r, 0))
+
+  if (all_p) {
+    r.topLeft.x = 0; r.topLeft.y = 0; r.extent.x = ww; r.extent.y = wh;
+    RECT();
+    return;
+  }
+
+  /* Color in these four rectangles:
+     ---------------
+     |______A______|
+     |_B_|_____|_C_|
+     |______D______|
+   */
+
+  r.topLeft.x = 0; r.topLeft.y = 0;  r.extent.x = ww; r.extent.y = dy; /* A */
+  RECT();
+
+  r.topLeft.x = 0; r.topLeft.y = dy; r.extent.x = dx; r.extent.y = dh; /* B */
+  RECT();
+
+  r.topLeft.x = dx + dw; r.topLeft.y = dy;			       /* C */
+  r.extent.x = ww - r.topLeft.x;
+  r.extent.y = dh;
+  RECT();
+
+  r.topLeft.x = 0; r.topLeft.y = dy + dh;			       /* D */
+  r.extent.x = ww;
+  r.extent.y = wh - r.topLeft.y;
+  RECT();
+# undef RECT
 }
 
 
-static void draw_clock (struct state *state, Boolean colonic_p);
+static void draw_clock (struct state *state);
 static void cycle_colors (struct state *, Boolean init_p);
 
 static void
@@ -619,7 +698,7 @@ start_sequence (struct state *state)
       copy_frame (state, state->current_frames[i], state->orig_frames[i]);
     }
 
-  draw_clock (state, true);
+  draw_clock (state);
   fill_target_digits (state);
 
   for (i = 0; i < countof (state->current_frames); i++)
@@ -700,13 +779,11 @@ tick_sequence (struct state *state)
       start_sequence (state);
       state->tick = state->ticks;
     }
-
-  /* cycle_colors (state, false); */
 }
 
 
 static void
-draw_clock (struct state *state, Boolean colonic_p)
+draw_clock (struct state *state)
 {
   int x, y, i;
   int width, height;
@@ -728,10 +805,9 @@ draw_clock (struct state *state, Boolean colonic_p)
   x += state->char_width
 
 # define COLON() \
-  if (colonic_p) \
-    draw_frame (state, state->base_frames \
-                         [state->display_date == DTime ? 10 : 11], \
-                x, y); \
+  draw_frame (state, state->base_frames \
+                       [state->display_date == DTime ? 10 : 11], \
+              x, y); \
   x += state->colon_width
 
   i = 0;
@@ -758,9 +834,9 @@ draw_clock (struct state *state, Boolean colonic_p)
 
 
 static void
-set_colors (struct state *state)
+init_colors (struct state *state)
 {
-  if (!state->color_p) return;
+  if (!state->color_p) goto SKIP;
 
   if (state->rom_version < ROM_35)
     return;
@@ -778,6 +854,7 @@ set_colors (struct state *state)
   rgb_to_hsv ( state->bg.r,  state->bg.g,  state->bg.b,
               &state->bg.h, &state->bg.s, &state->bg.v);
 
+ SKIP:
   cycle_colors (state, true);
 }
 
@@ -788,7 +865,7 @@ cycle_colors (struct state *state, Boolean init_p)
   RGBColorType table[2];
   int hue_tick = 2;
 
-  if (!state->color_p) return;
+  if (!state->color_p) goto SKIP;
 
   if (state->rom_version < ROM_35)
     return;
@@ -818,9 +895,11 @@ cycle_colors (struct state *state, Boolean init_p)
   table[1].b = state->bg.b >> 8;
 
   WinPalette (winPaletteSet, FG_PIXEL, 2, table);
-
-  WinSetForeColor (FG_PIXEL);   /* probably redundant? */
+  WinSetForeColor (FG_PIXEL);
   WinSetBackColor (BG_PIXEL);
+
+ SKIP:
+  clear_borders (state, init_p);
 }
 
 
@@ -948,13 +1027,14 @@ do_settings_form (struct state *state, FormPtr form)
     prefs.ticks = state->ticks;
 
 #ifdef HAVE_PREFSETAPPPREFERENCES_20
-    PrefSetAppPreferences (CREATOR_ID, 0, 1, &prefs, sizeof(prefs), true);
+    PrefSetAppPreferences (CREATOR_ID, PREFS_VERSION, 1,
+                           &prefs, sizeof(prefs), true);
 #else  /* !HAVE_PREFSETAPPPREFERENCES_20 */
     PrefSetAppPreferences (CREATOR_ID, 1, &prefs, sizeof(prefs));
 #endif /* !HAVE_PREFSETAPPPREFERENCES_20 */
   }
 
-  set_colors (state);
+  init_colors (state);
 
   return true;
 }
@@ -1015,8 +1095,7 @@ app_handle_event (struct state *state, EventPtr event)
         {
           /* Display date of any of the hardware keys (arrows, center-button)
              are pressed or held down. */
-          if (!state->seconds_only_p)
-            state->display_date = DDateIn;
+          state->display_date = DDateIn;
           return true;
         }
       else
@@ -1025,8 +1104,7 @@ app_handle_event (struct state *state, EventPtr event)
 
     case penDownEvent:		    /* Display date while pen down */
       state->pen_down_p = true;
-      if (!state->seconds_only_p)
-        state->display_date = DDateIn;
+      state->display_date = DDateIn;
       return true;
       break;
 
@@ -1035,10 +1113,25 @@ app_handle_event (struct state *state, EventPtr event)
       return true;
       break;
 
+    case winEnterEvent:
+      if (event->data.winEnter.enterWindow == state->window)
+        state->active_window_p = 1;
+      return false;
+      break;
+
+    case winExitEvent:
+      if (event->data.winEnter.exitWindow == state->window)
+        state->active_window_p = 0;
+      return false;
+      break;
+
     case nilEvent:
       {
         Boolean redraw_p = state->redraw_p;
 
+        if (! state->active_window_p)
+          return true;
+
         tick_sequence (state);
 
         if (redraw_p)
@@ -1047,11 +1140,10 @@ app_handle_event (struct state *state, EventPtr event)
             state->redraw_p = false;
             for (i = 0; i < countof(state->target_digits); i++)
               state->target_digits[i] = state->current_digits[i] = -2;
-            set_colors (state);
-            draw_background (state);
+            init_colors (state);
           }
 
-        draw_clock (state, false);
+        draw_clock (state);
 
         if (redraw_p)
           sync_to_seconds ();
@@ -1113,15 +1205,17 @@ initialize (struct state *state)
   prefs_size = sizeof(prefs);
 
 #ifdef HAVE_PREFSETAPPPREFERENCES_20
-  got_prefs = (PrefGetAppPreferences (CREATOR_ID, 0, &prefs, &prefs_size,
-                                      true) != noPreferenceFound &&
-               prefs_size >= sizeof(prefs));
+  got_prefs = (PREFS_VERSION ==
+               PrefGetAppPreferences (CREATOR_ID, PREFS_VERSION,
+                                      &prefs, &prefs_size,
+                                      true));
 #else  /* !HAVE_PREFSETAPPPREFERENCES_20 */
   got_prefs = (PrefGetAppPreferences (CREATOR_ID, 1, &prefs, prefs_size));
 #endif /* !HAVE_PREFSETAPPPREFERENCES_20 */
 
 
   state->inverted_p = true;
+  state->seconds_only_p = 0;
   state->ticks = DEFAULT_FRAME_RATE;
 
   if (got_prefs)
@@ -1137,7 +1231,6 @@ initialize (struct state *state)
   state->test_hack = 0;
   state->interval = SysTicksPerSecond() / state->ticks;
   state->tick = state->ticks;
-  state->seconds_only_p = 0;
 
   WinGetWindowExtent (&state->win_width, &state->win_height);
 
@@ -1171,12 +1264,14 @@ start_app (struct state *state)
   FrmSetActiveForm (form);
   FrmDrawForm (form);
 
+  state->window = WinGetDrawWindow();
+
   if (state->rom_version < ROM_30)
     state->color_p = false;
   else
     WinScreenMode (winScreenModeGet, NULL, NULL, NULL, &state->color_p);
 
-  set_colors (state);
+  init_colors (state);
 
   return false;
 }
