@@ -1,5 +1,5 @@
 /* Dali Clock - a melting digital clock for PalmOS.
- * Copyright (c) 1991-2006 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2007 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -13,10 +13,6 @@
 #ifndef __NUMBERS_H__
 #define __NUMBERS_H__
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
 typedef unsigned short POS;
 
 struct raw_number {
